@@ -1,8 +1,8 @@
-import datetime
 import getpass
 import logging
 import os
 import re
+import subprocess
 import sys
 import textwrap
 import xmlrpclib
@@ -42,7 +42,8 @@ VERSION_RE = re.compile(r'(?P<MAJOR>\d+)\.(?P<MINOR>\d+)(?:\.(?P<MICRO>\d+))?')
 # NOTE: The MICRO version number is appended to the format later, since it is
 # optional if the micro format is 0
 SEARCH_VERSION_RE_FORMAT = (r'(?P<prefix>v|V|[vV]ersion\s+|pyfits-)'
-                             '%(major)s\.%(minor)s(?:\s*\((?P<date>.+)\))?')
+                             '%(major)s\.%(minor)s'
+                             '(?:\s*\((?P<date>[A-Za-z]+ \d{2} \d{4})\))?')
 
 DATE_FORMAT = '%B %d %Y'
 
@@ -55,7 +56,7 @@ class ReleaseManager(object):
         self.released_version = ''
 
     def prereleaser_before(self, data):
-        """Set tag_svn_revision to False."""
+        """Check the long-description."""
 
         if data['name'] != 'pyfits':
             return
@@ -63,17 +64,12 @@ class ReleaseManager(object):
         global log
         log = logging.getLogger('prerelease')
 
-        # TODO: This bit should belong to a generic release hook somewhere in
-        # the stsci namespace, or even should maybe be suggested as a built in
-        # action of zest.releaser
-        def callback(section, option, value, lineno, line):
-            if section == 'egg_info' and option == 'tag_svn_revision':
-                log.info('Disabling tag_svn_revision in setup.cfg')
-                return 'tag_svn_revision = False\n'
-            else:
-                return line
+        if not check_long_description():
+            log.error('Errors rendering the long description rst. Run '
+                      'CHANGES.txt and README.txt through rst2html.py to '
+                      'find the errors and correct them.')
+            sys.exit(1)
 
-        config_parser('setup.cfg', callback)
 
     def prereleaser_middle(self, data):
         """Update the Sphinx conf.py"""
@@ -85,7 +81,7 @@ class ReleaseManager(object):
         cfg = ConfigParser()
         cfg.read('setup.cfg')
         authors = cfg.get('metadata', 'author')
-        authors = [a.strip() for a in authors.split(', ')]
+        authors = [a.strip() for a in authors.split(',')]
 
         update_docs_config(data['new_version'], authors)
 
@@ -109,23 +105,6 @@ class ReleaseManager(object):
 
         self.released_version = data['version']
 
-    def postreleaser_before(self, data):
-        """Restore tag_svn_revision"""
-
-        if data['name'] != 'pyfits':
-            return
-
-        global log
-        log = logging.getLogger('postrelease')
-
-        def callback(section, option, value, lineno, line):
-            if section == 'egg_info' and option == 'tag_svn_revision':
-                return 'tag_svn_revision = True'
-            else:
-                return line
-
-        config_parser('setup.cfg', callback)
-
     def postreleaser_after(self, data):
         """
         Used to update the PyFITS website.
@@ -168,7 +147,7 @@ class ReleaseManager(object):
         if not match:
             log.error('Previous version (%s) is invalid: Version must be in '
                       'the MAJOR.MINOR[.MICRO] format.' % previous_version)
-            sys.exit()
+            sys.exit(1)
 
         micro = int(match.group('MICRO')) if match.group('MICRO') else 0
 
@@ -179,7 +158,7 @@ class ReleaseManager(object):
         if not match:
             log.error('New version (%s) is invalid: Version must be in '
                       'the MAJOR.MINOR[.MICRO] format.' % new_version)
-            sys.exit()
+            sys.exit(1)
 
         micro = int(match.group('MICRO')) if match.group('MICRO') else 0
 
@@ -201,7 +180,7 @@ class ReleaseManager(object):
         def version_replace(match):
             repl = match.group('prefix') + new_version_str
             if match.group('date'):
-                today = datetime.datetime.today().strftime(DATE_FORMAT)
+                today = datetime.today().strftime(DATE_FORMAT)
                 repl += ' (%s)' % today
             return repl
 
@@ -227,6 +206,38 @@ class ReleaseManager(object):
             pass
 
 
+def check_long_description():
+    """The long-description metadata is created by combining the README.txt
+    and CHANGES.txt files which should be a valid rst document.
+
+    Try running this through rst2html to make sure the markup at least
+    renders, because this will also be used to render the description
+    on PyPI.
+    """
+
+    setup_py = subprocess.Popen([sys.executable, 'setup.py',
+                                 '--long-description'],
+                                 stdout=subprocess.PIPE)
+    stdout, _ = setup_py.communicate()
+
+    if setup_py.returncode == 0:
+        try:
+            rst2html = subprocess.Popen(['rst2html.py', '--halt=2'],
+                                        stdout=subprocess.PIPE,
+                                        stdin=subprocess.PIPE,
+                                        stderr=subprocess.PIPE)
+            rst2html.communicate(stdout)
+        except OSError, e:
+            log.error('Error running rst2html.py: %s\n'
+                      'Make sure you have docutils correctly installed.' % e)
+            sys.exit(1)
+
+        if rst2html.returncode == 0:
+            return True
+
+    return False
+
+
 def update_docs_config(new_version, authors):
     """
     Updates the conf.py for the Sphinx documentation with the new version
