@@ -19,7 +19,7 @@ from pyfits.hdu.table import _TableBaseHDU
 from pyfits.util import (Extendable, _is_int, _tmp_name, _with_extensions,
                          _pad_length, BLOCK_SIZE, isfile, fileobj_name,
                          fileobj_closed, fileobj_mode, ignore_sigint,
-                         _get_array_memmap, indent)
+                         _get_array_mmap, indent)
 from pyfits.verify import _Verify, _ErrList, VerifyError
 
 
@@ -512,7 +512,9 @@ class HDUList(list, _Verify):
             name = hdu.name
             if isinstance(name, str):
                 name = name.strip().upper()
-            if name == _key and (_ver is None or _ver == hdu._extver):
+            # 'PRIMARY' should always work as a reference to the first HDU
+            if ((name == _key or (_key == 'PRIMARY' and idx == 0)) and
+                (_ver is None or _ver == hdu._extver)):
                 found = idx
                 nfound += 1
 
@@ -888,7 +890,7 @@ class HDUList(list, _Verify):
             if sys.platform.startswith('win'):
                 # Collect a list of open mmaps to the data; this well be used
                 # later.  See below.
-                mmaps = [(idx, _get_array_memmap(hdu.data), hdu.data)
+                mmaps = [(idx, _get_array_mmap(hdu.data), hdu.data)
                          for idx, hdu in enumerate(self) if hdu._data_loaded]
 
             hdulist.__file.close()
@@ -898,9 +900,9 @@ class HDUList(list, _Verify):
                 # Close all open mmaps to the data.  This is only necessary on
                 # Windows, which will not allow a file to be renamed or deleted
                 # until all handles to that file have been closed.
-                for idx, map, arr in mmaps:
-                    if map is not None:
-                        map.base.close()
+                for idx, mmap, arr in mmaps:
+                    if mmap is not None:
+                        mmap.close()
 
             os.remove(self.__file.name)
 
@@ -921,7 +923,7 @@ class HDUList(list, _Verify):
                 # Need to update the _file attribute and close any open mmaps
                 # on each HDU
                 if (hdu._data_loaded and
-                    _get_array_memmap(hdu.data) is not None):
+                    _get_array_mmap(hdu.data) is not None):
                     del hdu.data
                 hdu._file = ffo
 
