@@ -61,9 +61,22 @@ class _TableLikeHDU(_ValidHDU):
         names = [n for idx, n in enumerate(columns.names)
                  if not columns[idx]._phantom]
         dtype = np.rec.format_parser(formats, names, None)._descr
-        raw_data = self._file.readarray(offset=self._datLoc, dtype=dtype,
-                                        shape=columns._shape)
-        data = raw_data.view(np.rec.recarray)
+
+        # TODO: Details related to variable length arrays need to be dealt with
+        # specifically in the BinTableHDU class, since they're a detail
+        # specific to FITS binary tables
+        if (_FormatP in [type(r) for r in recformats] and
+            self._datSpan > self._theap):
+            # We have a heap; include it in the raw_data
+            raw_data = self._file.readarray(offset=self._datLoc, dtype=np.byte,
+                                            shape=(self._datSpan))
+            data = raw_data[:self._theap].view(dtype=dtype,
+                                               type=np.rec.recarray)
+        else:
+            raw_data = self._file.readarray(offset=self._datLoc, dtype=dtype,
+                                            shape=(columns._shape,))
+            data = raw_data.view(np.rec.recarray)
+
         self._init_tbdata(data)
         return data.view(self._data_type)
 
@@ -73,7 +86,8 @@ class _TableLikeHDU(_ValidHDU):
         data.dtype = data.dtype.newbyteorder('>')
 
         # pass datLoc, for P format
-        data._heapoffset = self._theap + self._datLoc
+        data._heapoffset = self._theap
+        data._heapsize = self._header['PCOUNT']
         data._file = self._file
         tbsize = self._header['NAXIS1'] * self._header['NAXIS2']
         data._gap = self._theap - tbsize
@@ -1252,7 +1266,29 @@ def new_table(input, header=None, nrows=0, fill=False, tbtype='BinTableHDU'):
                             arr += bzero
                         hdu.data._convert[idx][:n] = arr[:n]
                 else:
-                    field[:n] = arr[:n]
+                    outarr = field[:n]
+                    inarr = arr[:n]
+                    if inarr.shape != outarr.shape:
+                        if inarr.dtype != outarr.dtype:
+                            inarr = inarr.view(outarr.dtype)
+
+                        # This is a special case to handle input arrays with
+                        # non-trivial TDIMn.
+                        # By design each row of the outarray is 1-D, while each
+                        # row of the input array may be n-D
+                        if outarr.ndim > 1:
+                            # The normal case where the first dimension is the
+                            # rows
+                            inarr_rowsize = inarr[0].size
+                            inarr = inarr.reshape((n, inarr_rowsize))
+                            outarr[:,:inarr_rowsize] = inarr
+                        else:
+                            # Special case for strings where the out array only
+                            # has one dimension (the second dimension is rolled
+                            # up into the strings
+                            outarr[:n] = inarr.ravel()
+                    else:
+                        field[:n] = arr[:n]
 
         if n < nrows:
             # If there are additional rows in the new table that were not
