@@ -2,6 +2,7 @@ from __future__ import division # confidence high
 
 import os
 import shutil
+import stat
 import tempfile
 import warnings
 
@@ -20,8 +21,20 @@ class PyfitsTestCase(object):
     def teardown(self):
         shutil.rmtree(self.temp_dir)
 
+    def copy_file(self, filename):
+        """Copies a backup of a test data file to the temp dir and sets its
+        mode to writeable.
+        """
+
+        shutil.copy(self.data(filename), self.temp(filename))
+        os.chmod(self.temp(filename), stat.S_IREAD | stat.S_IWRITE)
+
     def data(self, filename):
+        """Returns the path to a test data file."""
+
         return os.path.join(self.data_dir, filename)
 
     def temp(self, filename):
+        """ Returns the full path to a file in the test temp dir."""
+
         return os.path.join(self.temp_dir, filename)
