@@ -1,4 +1,4 @@
-from __future__ import division # confidence high
+from __future__ import division  # confidence high
 from __future__ import with_statement
 
 import gzip
@@ -11,7 +11,7 @@ import numpy as np
 import pyfits
 from pyfits.convenience import _getext
 from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import catch_warnings, BytesIO, CaptureStdout
+from pyfits.tests.util import catch_warnings, BytesIO, ignore_warnings
 
 from nose.tools import assert_equal, assert_raises, assert_true, assert_false
 
@@ -21,13 +21,17 @@ class TestCore(PyfitsTestCase):
         with pyfits.open(self.data('ascii.fits')) as f:
             pass
 
+    def test_missing_file(self):
+        assert_raises(IOError, pyfits.open, self.temp('does-not-exist.fits'))
+
     def test_naxisj_check(self):
         hdulist = pyfits.open(self.data('o4sp040b0_raw.fits'))
 
         hdulist[1].header.update("NAXIS3", 500)
 
         assert 'NAXIS3' in hdulist[1].header
-        hdulist.verify('fix')
+        with ignore_warnings():
+            hdulist.verify('fix')
         assert 'NAXIS3' not in hdulist[1].header
 
     def test_byteswap(self):
@@ -343,13 +347,6 @@ class TestFileFunctions(PyfitsTestCase):
 
         assert_equal(len(pyfits.open(self._make_gzip_file('test0.fz'))), 5)
 
-    def test_open_gzipped_writeable(self):
-        """Opening gzipped files in a writeable mode should fail."""
-
-        gf = self._make_gzip_file()
-        assert_raises(IOError, pyfits.open, gf, 'update')
-        assert_raises(IOError, pyfits.open, gf, 'append')
-
     def test_open_zipped(self):
         assert_equal(len(pyfits.open(self._make_zip_file())), 5)
 
@@ -401,6 +398,17 @@ class TestFileFunctions(PyfitsTestCase):
         finally:
             gf.close()
 
+    def test_open_gzip_file_for_writing(self):
+        """Regression test for #195."""
+
+        gf = self._make_gzip_file()
+        with pyfits.open(gf, mode='update') as h:
+            h[0].header['EXPFLAG'] = 'ABNORMAL'
+        with pyfits.open(gf) as h:
+            # Just to make sur ethe update worked; if updates work
+            # normal writes should work too...
+            assert_equal(h[0].header['EXPFLAG'], 'ABNORMAL')
+
     def test_read_file_like_object(self):
         """Test reading a FITS file from a file-like object."""
 
@@ -511,7 +519,7 @@ class TestStreamingFunctions(PyfitsTestCase):
 
         assert_raises(pyfits.VerifyError, hdul.writeto, self.temp('temp.fits'),
                       output_verify='exception')
-        with CaptureStdout():
+        with ignore_warnings():
             hdul.writeto(self.temp('temp.fits'), output_verify='fix')
         with pyfits.open(self.temp('temp.fits')):
             assert_equal(hdul[1].name, '12345678')
