@@ -2,7 +2,6 @@ from __future__ import division  # confidence high
 from __future__ import with_statement
 
 import os
-import shutil
 import time
 import warnings
 
@@ -10,10 +9,11 @@ import numpy as np
 
 import pyfits
 from pyfits.tests import PyfitsTestCase
-from pyfits.tests.util import CaptureStdout, catch_warnings, ignore_warnings
+from pyfits.tests.util import catch_warnings, ignore_warnings
 
 from nose.tools import (assert_equal, assert_raises, assert_true, assert_false,
                         assert_not_equal)
+from numpy.testing import assert_allclose
 
 
 class TestImageFunctions(PyfitsTestCase):
@@ -105,7 +105,7 @@ class TestImageFunctions(PyfitsTestCase):
                          "ABC     = (1.23453774378878E+88, 6.32476736476374E-15)                          ")
 
     def test_card_image_constructed_too_long(self):
-        with CaptureStdout():
+        with ignore_warnings():
             # card image constructed from key/value/comment is too long
             # (non-string value)
             c = pyfits.Card('abc', 9, 'abcde'*20)
@@ -218,19 +218,22 @@ class TestImageFunctions(PyfitsTestCase):
     def test_verification(self):
         # verification
         c = pyfits.Card.fromstring('abc= a6')
-        with CaptureStdout() as f:
+        with catch_warnings(record=True) as w:
             c.verify()
+            assert_equal(len(w), 1)
             assert_true(
                 'Card image is not FITS standard (equal sign not at column 8).'
-                in f.getvalue())
+                in str(w[0].message))
         assert_equal(str(c),
                      "abc= a6                                                                         ")
 
     def test_fix(self):
         c = pyfits.Card.fromstring('abc= a6')
-        with CaptureStdout() as f:
+        with catch_warnings(record=True) as w:
             c.verify('fix')
-            assert_true('Fixed card to be FITS standard.: ABC' in f.getvalue())
+            assert_equal(len(w), 1)
+            assert_true('Fixed card to be FITS standard.: ABC' in
+                        str(w[0].message))
         assert_equal(str(c),
                      "ABC     = 'a6      '                                                            ")
 
@@ -496,18 +499,21 @@ class TestImageFunctions(PyfitsTestCase):
     def test_verification_on_output(self):
         # verification on output
         # make a defect HDUList first
-        with CaptureStdout() as f:
+        with catch_warnings(record=True) as w:
             x = pyfits.ImageHDU()
             hdu = pyfits.HDUList(x) # HDUList can take a list or one single HDU
             hdu.verify()
+            assert_equal(len(w), 1)
             assert_true(
-                "HDUList's 0th element is not a primary HDU." in f.getvalue())
+                "HDUList's 0th element is not a primary HDU." in
+                str(w[0].message))
 
-        with CaptureStdout() as f:
+        with catch_warnings(record=True) as w:
             hdu.writeto(self.temp('test_new2.fits'), 'fix')
+            assert_equal(len(w), 1)
             assert_true(
                 "HDUList's 0th element is not a primary HDU.  "
-                "Fixed by inserting one as 0th HDU." in f.getvalue())
+                "Fixed by inserting one as 0th HDU." in str(w[0].message))
 
     def test_section(self):
         # section testing
@@ -852,7 +858,7 @@ class TestImageFunctions(PyfitsTestCase):
         """
 
         # Copy the original file before making any possible changes to it
-        shutil.copy(self.data('comp.fits'), self.temp('comp.fits'))
+        self.copy_file('comp.fits')
         mtime = os.stat(self.temp('comp.fits')).st_mtime
 
         time.sleep(1)
@@ -916,7 +922,8 @@ class TestImageFunctions(PyfitsTestCase):
 
         hdul = pyfits.open(self.data('fixed-1890.fits'))
         orig_data = hdul[0].data
-        hdul.writeto(self.temp('test_new.fits'), clobber=True)
+        with ignore_warnings():
+            hdul.writeto(self.temp('test_new.fits'), clobber=True)
         hdul.close()
         hdul = pyfits.open(self.temp('test_new.fits'))
         assert_true((hdul[0].data == orig_data).all())
@@ -925,7 +932,8 @@ class TestImageFunctions(PyfitsTestCase):
         # Just as before, but this time don't touch hdul[0].data before writing
         # back out--this is the case that failed in #84
         hdul = pyfits.open(self.data('fixed-1890.fits'))
-        hdul.writeto(self.temp('test_new.fits'), clobber=True)
+        with ignore_warnings():
+            hdul.writeto(self.temp('test_new.fits'), clobber=True)
         hdul.close()
         hdul = pyfits.open(self.temp('test_new.fits'))
         assert_true((hdul[0].data == orig_data).all())
@@ -1005,14 +1013,14 @@ class TestImageFunctions(PyfitsTestCase):
         """
 
         # Copy the original file before saving to it
-        shutil.copy(self.data('test0.fits'), self.temp('test_new.fits'))
-        with pyfits.open(self.temp('test_new.fits'), mode='update') as hdul:
+        self.copy_file('test0.fits')
+        with pyfits.open(self.temp('test0.fits'), mode='update') as hdul:
             orig_data = hdul[1].data.copy()
             hdr_copy = hdul[1].header.copy()
             del hdr_copy['NAXIS*']
             hdul[1].header = hdr_copy
 
-        with pyfits.open(self.temp('test_new.fits')) as hdul:
+        with pyfits.open(self.temp('test0.fits')) as hdul:
             assert_true((orig_data == hdul[1].data).all())
 
     def test_open_scaled_in_update_mode(self):
@@ -1026,7 +1034,7 @@ class TestImageFunctions(PyfitsTestCase):
         """
 
         # Copy the original file before making any possible changes to it
-        shutil.copy(self.data('scale.fits'), self.temp('scale.fits'))
+        self.copy_file('scale.fits')
         mtime = os.stat(self.temp('scale.fits')).st_mtime
 
         time.sleep(1)
@@ -1117,3 +1125,25 @@ class TestImageFunctions(PyfitsTestCase):
         assert_equal(hdul[1].header['BITPIX'], -32)
         assert_true('BZERO' not in hdul[1].header)
         assert_true('BSCALE' not in hdul[1].header)
+
+    def test_compression_column_tforms(self):
+        """Regression test for #199."""
+
+        # Some interestingly tiled data so that some of it is quantized and
+        # some of it ends up just getting gzip-compressed
+        data2 = ((np.arange(1, 8, dtype=np.float32) * 10)[:, np.newaxis] +
+                np.arange(1, 7))
+        np.random.seed(1337)
+        data1 = np.random.uniform(size=(6 * 4, 7 * 4))
+        data1[:data2.shape[0], :data2.shape[1]] = data2
+        chdu = pyfits.CompImageHDU(data1, compressionType='RICE_1',
+                                   tileSize=(6, 7))
+        chdu.writeto(self.temp('test.fits'))
+
+        with pyfits.open(self.temp('test.fits'),
+                         disable_image_compression=True) as h:
+            assert_equal(h[1].header['TFORM1'], '1PB(51)')
+            assert_equal(h[1].header['TFORM2'], '1PD(2)')
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            assert_allclose(h[1].data, data1, rtol=0.1, atol=0.1)
