@@ -360,7 +360,7 @@ class TestTableFunctions(PyfitsTestCase):
                                (3,'Rigil Kent',-0.1,'G2V')],
                               formats='int16,a20,float32,a10',
                               names='order,name,mag,Sp')
-        hdu=pyfits.new_table(bright,nrows=2,tbtype='TableHDU')
+        hdu = pyfits.new_table(bright, nrows=2, tbtype='TableHDU')
 
         # Verify that all ndarray objects within the HDU reference the
         # same ndarray.
@@ -433,13 +433,14 @@ class TestTableFunctions(PyfitsTestCase):
             assert_equal(hdul[1].data[1][3], 'F0Ib')
         del hdul
 
-        hdu = pyfits.new_table(bright,nrows=2)
-        tmp = np.rec.array([(1,'Serius',-1.45,'A1V'),
-                            (2,'Canopys',-0.73,'F0Ib')],
+        hdu = pyfits.new_table(bright, nrows=2)
+        tmp = np.rec.array([(1, 'Serius', -1.45,'A1V'),
+                            (2, 'Canopys', -0.73,'F0Ib')],
                             formats='int16,a20,float32,a10',
                             names='order,name,mag,Sp')
         assert_true(comparerecords(hdu.data, tmp))
-        hdu.writeto(self.temp('toto.fits'), clobber=True)
+        with ignore_warnings():
+            hdu.writeto(self.temp('toto.fits'), clobber=True)
         with pyfits.open(self.temp('toto.fits')) as hdul:
             assert_true(comparerecords(hdu.data, hdul[1].data))
 
@@ -1612,7 +1613,8 @@ class TestTableFunctions(PyfitsTestCase):
         del hdul
 
         ahdu = pyfits.new_table([acol], tbtype='TableHDU')
-        ahdu.writeto(self.temp('newtable.fits'), clobber=True)
+        with ignore_warnings():
+            ahdu.writeto(self.temp('newtable.fits'), clobber=True)
 
         with pyfits.open(self.temp('newtable.fits')) as hdul:
             assert_equal(hdul[1].data.tostring().decode('raw-unicode-escape'),
@@ -1623,7 +1625,8 @@ class TestTableFunctions(PyfitsTestCase):
 
         # Now serialize once more as a binary table; padding bytes should
         # revert to zeroes
-        ahdu.writeto(self.temp('newtable.fits'), clobber=True)
+        with ignore_warnings():
+            ahdu.writeto(self.temp('newtable.fits'), clobber=True)
         with pyfits.open(self.temp('newtable.fits')) as hdul:
             assert_equal(hdul[1].data.tostring().decode('raw-unicode-escape'),
                          s)
@@ -1644,7 +1647,6 @@ class TestTableFunctions(PyfitsTestCase):
         # Modify the TDIM fields to my own specification
         thdu.header.update('TDIM1', '(2,3)')
         thdu.header.update('TDIM2', '(4,2)')
-
         thdu.writeto(self.temp('newtable.fits'))
 
         with pyfits.open(self.temp('newtable.fits')) as hdul:
@@ -1670,7 +1672,8 @@ class TestTableFunctions(PyfitsTestCase):
         data = np.zeros(3, dtype=[('x', 'f4'), ('s', 'S5', 4)])
         data['x'] = 1, 2, 3
         data['s'] = 'ok'
-        pyfits.writeto(self.temp('newtable.fits'), data, clobber=True)
+        with ignore_warnings():
+            pyfits.writeto(self.temp('newtable.fits'), data, clobber=True)
 
         t = pyfits.getdata(self.temp('newtable.fits'))
 
@@ -1693,6 +1696,97 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(t.field(1).dtype.str[-1], '5')
         assert_equal(t.field(1).shape, (3, 4, 3))
 
+    def test_string_array_round_trip(self):
+        """Regression test for #201."""
+
+        data = [['abc', 'def', 'ghi'],
+                ['jkl', 'mno', 'pqr'],
+                ['stu', 'vwx', 'yz ']]
+
+        recarr = np.rec.array([(data,), (data,)], formats=['(3,3)S3'])
+
+        t = pyfits.BinTableHDU(data=recarr)
+        t.writeto(self.temp('test.fits'))
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            assert_true('TDIM1' in h[1].header)
+            assert_equal(h[1].header['TDIM1'], '(3,3,3)')
+            assert_equal(len(h[1].data), 2)
+            assert_equal(len(h[1].data[0]), 1)
+            assert_true((h[1].data.field(0)[0] ==
+                         recarr.field(0)[0].decode('ascii')).all())
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            # Access the data; I think this is necessary to exhibit the bug
+            # reported in #201
+            h[1].data[:]
+            h.writeto(self.temp('test2.fits'))
+
+        with pyfits.open(self.temp('test2.fits')) as h:
+            assert_true('TDIM1' in h[1].header)
+            assert_equal(h[1].header['TDIM1'], '(3,3,3)')
+            assert_equal(len(h[1].data), 2)
+            assert_equal(len(h[1].data[0]), 1)
+            assert_true((h[1].data.field(0)[0] ==
+                         recarr.field(0)[0].decode('ascii')).all())
+
+    def test_new_table_with_nd_column(self):
+        """Regression test for
+        https://github.com/spacetelescope/PyFITS/issues/3
+        """
+
+        arra = np.array(['a', 'b'], dtype='|S1')
+        arrb = np.array([['a', 'bc'], ['cd', 'e']], dtype='|S2')
+        arrc = np.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]])
+
+        cols = [
+            pyfits.Column(name='str', format='1A', array=arra),
+            pyfits.Column(name='strarray', format='4A', dim='(2,2)',
+                          array=arrb),
+            pyfits.Column(name='intarray', format='4I', dim='(2, 2)',
+                          array=arrc)
+        ]
+
+        hdu = pyfits.new_table(pyfits.ColDefs(cols))
+        hdu.writeto(self.temp('test.fits'))
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            # Need to force string arrays to byte arrays in order to compare
+            # correctly on Python 3
+            assert_true((h[1].data['str'].encode('ascii') == arra).all())
+            assert_true((h[1].data['strarray'].encode('ascii') == arrb).all())
+            assert_true((h[1].data['intarray'] == arrc).all())
+
+    def test_mismatched_tform_and_tdim(self):
+        """Normally the product of the dimensions listed in a TDIMn keyword
+        must be less than or equal to the repeat count in the TFORMn keyword.
+
+        This tests that this works if less than (treating the trailing bytes
+        as unspecified fill values per the FITS standard) and fails if the
+        dimensions specified by TDIMn are greater than the repeat count.
+        """
+
+        arra = np.array([[[1, 2], [3, 4]], [[5, 6], [7, 8]]])
+        arrb = np.array([[[9, 10], [11, 12]], [[13, 14], [15, 16]]])
+
+        cols = [pyfits.Column(name='a', format='20I', dim='(2,2)',
+                              array=arra),
+                pyfits.Column(name='b', format='4I', dim='(2,2)',
+                              array=arrb)]
+
+        # The first column has the mismatched repeat count
+        hdu = pyfits.new_table(pyfits.ColDefs(cols))
+        hdu.writeto(self.temp('test.fits'))
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            assert_true((h[1].data['a'] == arra).all())
+            assert_true((h[1].data['b'] == arrb).all())
+
+        # If dims is more than the repeat count in the format specifier raise
+        # an error
+        assert_raises(ValueError, pyfits.Column, name='a', format='2I',
+                      dim='(2,2)', array=arra)
+
     def test_slicing(self):
         """Regression test for #52."""
 
@@ -1787,6 +1881,18 @@ class TestTableFunctions(PyfitsTestCase):
         assert_equal(thdu.columns.formats, ['L', 'L'])
         assert_true(comparerecords(data, array))
 
+    def test_table_from_bool_fields2(self):
+        """
+        Regression test for https://trac.assembla.com/pyfits/ticket/215
+
+        Tests the case where a multi-field ndarray (not a recarray) containing
+        a bool field is used to initialize a `BinTableHDU`.
+        """
+
+        arr = np.array([(False,), (True,), (False,)], dtype=[('a', '?')])
+        hdu = pyfits.BinTableHDU(data=arr)
+        assert_true((hdu.data['a'] == arr['a']).all())
+
     def test_bool_column_update(self):
         """Regression test for #139."""
 
@@ -1802,3 +1908,51 @@ class TestTableFunctions(PyfitsTestCase):
         with pyfits.open(self.temp('table.fits')) as hdul:
             assert_true((hdul[1].data['F1'] == [True, True]).all())
             assert_true((hdul[1].data['F2'] == [True, True]).all())
+
+    def test_missing_tnull(self):
+        """Regression test for #197."""
+
+        c = pyfits.Column('F1', 'A3', null='---',
+                          array=np.array(['1.0', '2.0', '---', '3.0']))
+        table = pyfits.new_table([c], tbtype='TableHDU')
+        table.writeto(self.temp('test.fits'))
+
+        # Now let's delete the TNULL1 keyword, making this essentially
+        # unreadable
+        with pyfits.open(self.temp('test.fits'), mode='update') as h:
+            h[1].header['TFORM1'] = 'E3'
+            del h[1].header['TNULL1']
+
+        with pyfits.open(self.temp('test.fits')) as h:
+            assert_raises(ValueError, lambda: h[1].data['F1'])
+
+        try:
+            with pyfits.open(self.temp('test.fits')) as h:
+                h[1].data['F1']
+        except ValueError, e:
+            assert_true(str(e).endswith(
+                         "the header may be missing the necessary TNULL1 "
+                         "keyword or the table contains invalid data"))
+
+    def test_getdata_vla(self):
+        """Regression test for #200."""
+        col = pyfits.Column(name='QUAL_SPE', format='PJ()',
+                            array=[np.arange(1572)]*225)
+        tb_hdu = pyfits.new_table([col])
+        pri_hdu = pyfits.PrimaryHDU()
+        hdu_list = pyfits.HDUList([pri_hdu,tb_hdu])
+        hdu_list.writeto(self.temp('toto.fits'))
+
+        data = pyfits.getdata(self.temp('toto.fits'))
+
+        # Need to compare to the original data row by row since the FITS_rec
+        # returns an array of _VLA objects
+        for row_a, row_b in zip(data['QUAL_SPE'], col.array):
+            assert_true((row_a == row_b).all())
+
+    def test_column_array_type_mismatch(self):
+        """Regression test for #218."""
+
+        arr = [-99] * 20
+        col = pyfits.Column('mag', format='E', array=arr)
+        assert_true((arr == col.array).all())
