@@ -569,6 +569,22 @@ def fileobj_closed(f):
         return True
 
 
+def fileobj_is_binary(f):
+    """
+    Returns True if the give file or file-like object has a file open in binary
+    mode.  When in doubt, returns True by default.
+    """
+
+    # TODO: In Python 3 it might be more reliable to check if the fileobj is a
+    # text reader or a binary reader
+
+    mode = fileobj_mode(f)
+    if mode:
+        return 'b' in mode
+    else:
+        return True
+
+
 def fileobj_mode(f):
     """
     Returns the 'mode' string of a file-like object if such a thing exists.
@@ -652,10 +668,7 @@ def _write_string(f, s):
 
     # Assume if the file object doesn't have a specific mode, that the mode is
     # binary
-    if hasattr(f, 'mode'):
-        binmode = 'b' in f.mode
-    else:
-        binmode = True
+    binmode = fileobj_is_binary(f)
 
     if binmode and isinstance(s, unicode):
         s = encode_ascii(s)
@@ -683,13 +696,17 @@ def _chunk_array(arr, CHUNK_SIZE=2 ** 25):
 def _convert_array(array, dtype):
     """
     Converts an array to a new dtype--if the itemsize of the new dtype is
-    the same as the old dtype, a view is returned.  Otherwise a new array must
-    be created.
+    the same as the old dtype and both types are not numeric, a view is
+    returned.  Otherwise a new array must be created.
     """
 
     if array.dtype == dtype:
         return array
-    elif array.dtype.itemsize == dtype.itemsize:
+    elif (array.dtype.itemsize == dtype.itemsize and not
+            (np.issubdtype(array.dtype, np.number) and
+             np.issubdtype(dtype, np.number))):
+        # Includes a special case when both dtypes are at least numeric to
+        # account for ticket #218: https://trac.assembla.com/pyfits/ticket/218
         return array.view(dtype)
     else:
         return array.astype(dtype)
